####################################################################################################################################################################################
###### Tool indicators ES Assesment V0 (31/03/2017)
###### Contact: Gudrun Schwilch <Gudrun.Schwilch@cde.unibe.ch>; Tatenda Lemann <tatenda.lemann@cde.unibe.ch>
###### Author: maria.ruiperezgonzalez@wur.nl; ISRIC - World Soil Information
###### Special attributions for the polarBarChart function. Adapted from:
## Original Polar Histogram by Christophe Ladroue
## Source: http://chrisladroue.com/2012/02/polar-histogram-pretty-and-useful/
## Modified from original by Christos Hatzis 3.22.2012 (CH)
#' CC licence: http://creativecommons.org/licenses/by-nc-sa/3.0/ 
#' Attribution - You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work).
#' Noncommercial - You may not use this work for commercial purposes.
#' Share Alike - If you alter, transform, or build upon this work, you may distribute the resulting work only under the same or similar license to this one. 
####################################################################################################################################################################################

####### FUNCTION 1: Prepare the data for the graph; the data is normalized to a range from 1 to 7.
prepareData <- function(dfc, level) {
 
 if(level=="plot level") {df <- dfc[3:6,1:15 ]}
 if(level=="wider area") {df <- dfc[9:12,1:15 ]}
 
 provservices <- df[1:2,1:5 ]
 provservices [3,]<-"Provisioning Services" 
 regservices <- df[c(1,3),6:12 ]
 regservices [3,]<-"Regulating Services" 
 cultservices <- df[c(1,4),  13:15]
 cultservices [3,]<-"Cultural Services" 
 
 df <- cbind(provservices, regservices, cultservices)
 
 df <- as.data.frame(t(df))
 colnames(df)<-c("item","value","family" )
 df$item <- as.character(df$item)
 df$value <- ifelse(
  df$value=="not relevant", -9999.0, as.numeric(as.character(df$value)))
 # df$value <- as.numeric(df$value)
 
 df$score<- NA
 df$score <- ifelse(
  df$value==0, "No change", ifelse(
   df$value==-9999.0,   "Not relevant", ifelse(
    df$value < 0, "Negative impact", ifelse(
     df$value > 0, "Positive impact", df$value)
   )
  )
 )
 
 df$value <- ifelse(
  df$value==-9999.0, 0.0, df$value)
 
 df[16,] <- df[11,];  df[17,] <- df[11,]
 df$value[16] <- 3 ;  df$value[17] <- -3
 df$score[16] <- "";  df$score[17] <- ""
 df$item[16] <- "z";  df$item[17] <- "z"
 
 # Normalized Data
 array <- df$value
 m = min(array); range = max(array) - m;
 array = (array - m) / range; x<- 1; y<- 7
 range2 = y - x;
 normalized = (array*range2) + x;
 
 df$value <- normalized
 
 return(df)
 
}

####### FUNCTION 2: runPlot Function (calls polarBarChart Function and prepareData Function)  
runPlot <- function(workingPath, nameExcelFile, time=time.lst[1], level=level.lst[1]) {
 
 setwd(workingPath)
 
 ## define variables
 time=time
 if(time=="short term") {nr=5}
 if(time=="long term") {nr=7}
 title=paste0(level, " : ", time)
 title.pdf=paste0(level, "_",time,"pdf")
 outname=paste0(level,"_level_",time)
 
 ## read data from excel
 data <- read_excel(paste0("./", nameExcelFile), sheet =nr)
 data <- data.frame(data)
 
 ## prepare the data and produce the graph
 if(level=="plot level"){ 
    dfcc = suppressWarnings(prepareData(dfc=data, level="plot level"))}else{ dfcc = suppressWarnings(prepareData(dfc=data, level="wider area"))} 
 
 p <- polarBarChart(
  df=dfcc,
  binSize=15,
  spaceBar=0.05,
  spaceItem=7,
  spaceFamily=9,
  innerRadius=0.5,
  outerRadius=7,
  nguides=5,
  alphaStart=0,
  circleProportion=0.8,
  familyLabels=TRUE,
  itemSize=10,
  legLabels=NULL,
  legTitle="Source",
  direction='outwards',
  
  ## here is possible to change the colors (see http://www.stat.columbia.edu/~tzheng/files/Rcolor.pdf)
  colNoChange =  "gold1", 
  colCircle0 = "whitesmoke", 
  colNotRelevant =  "cornsilk", 
  colPositiveImpact = "springgreen4",
  colNegativeImpact = "firebrick1", 
  title= title)
 
 print(p)
 
 ## print as pdf the single graphs
 name=paste0(time, "_", level, ".pdf")
 CairoPDF(file=name, width=22, height=22,   family="FreeSans")
 print(p)
 dev.off()
 return(p)
}

####### FUNCTION 3: polarBarChart Function
###### Adapted from:
## Original Polar Histogram by Christophe Ladroue
## Source: http://chrisladroue.com/2012/02/polar-histogram-pretty-and-useful/
## Modified from original by Christos Hatzis 3.22.2012 (CH)
#' CC licence: http://creativecommons.org/licenses/by-nc-sa/3.0/ 
#' Attribution - You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work).
#' License: Noncommercial - You may not use this work for commercial purposes.
#' v. 21022012

polarBarChart <- 
   function(
      df=NULL, 
      binSize=1, 
      spaceBar=0.05, 
      spaceItem=0.2, 
      spaceFamily=1.2,
      innerRadius=0.3,
      outerRadius=7,
      nguides=7,
      guides=pretty(range(c(1, df$value)), n=nguides, min.n=2),
      alphaStart=-0.3,
      circleProportion=0.8,
      direction="inwards",
      familyLabels=FALSE,
      itemSize=10,
      legLabels=NULL,
      legTitle="Source",
      colNoChange =  "gold1", 
      colCircle0 = "seashell4", 
      colNotRelevant =  "cornsilk", 
      colPositiveImpact = "springgreen4",
      colNegativeImpact = "firebrick1",
      title = NULL){
      
      
      
      require(ggplot2)
      require(plyr)
      
      guides=pretty(range(c(1, df$value)), n=5, min.n=2)
      # ordering df according to "family", "item" and "score"
      # df<-arrange(df,family,item,score)

      
      # family and item indices
      df$indexFamily <- as.integer(factor(df$family))
      # df$indexItem <- with(df, as.integer(factor(item, levels=item[!duplicated(item)])))        
      df$indexScore <- as.integer(factor(df$score))

      df<-arrange(df,family)
      # df <- rbind(df[df$item!="z", ],df[df$item=="z", ])
      
      # define the bins
      M <- nlevels(factor(df$score))
      vMax <- max(df$value)
      
      df$value <- df$value/vMax
      
      # linear projection
      # Switch function----
      # if set to "outwards", then the function "outwards" is used - otherwise the fnct "inwards"
      affine <- switch(direction,
                     'inwards'= function(y) (outerRadius-innerRadius)*y + innerRadius,
                     'outwards'=function(y) (outerRadius-innerRadius)*(1-y) + innerRadius,
                     stop(paste("Unknown direction")))
      
      df<-within(df, {
         # xmin is the how the items are placed on the x-axis
         xmin <-   c(0,50,100,250,300,350,400,450,600,650,700,750,800,850,900,950,1000)
         # xmin <- (indexScore - 1) * (binSize + spaceBar) + 
         #   (indexItem - 1) * (spaceItem + M * (binSize + spaceBar)) +
         #   (indexFamily - 1) * (spaceFamily - spaceItem)
         xmax <- xmin + binSize
         
         ymin <- affine(1)
         ymax <- affine(1 - value)
      }
      )     
      
      df <- df[1:15,]
      
      # build the guides
      guidesDF<-data.frame(
         xmin=rep(df$xmin,length(guides)),
         y=rep(guides/vMax,1,each=nrow(df)))
      
      guidesDF<-within(guidesDF,{
         xend<-xmin+binSize+spaceBar
         y<-affine(1-y)
      })
      
      # Generate data
      
      # Building the ggplot object
      totalLength <- tail(df$xmin + binSize + spaceBar + spaceFamily, 1)/circleProportion-0
      
      p<-ggplot()+geom_rect(
         aes(
            xmin=0,
            xmax=1112,
            ymin=0,
            ymax=4.257143,
            fill="no change")
      )  + labs(fill="Legend", shape="", size=14)
      
      p <- p+geom_rect(
         aes(
            xmin=df$xmin,
            xmax=df$xmax,
            ymin=df$ymin,
            ymax=df$ymax,
            fill=df$score)
      )
      
      # guides  
      p<-p+geom_segment(
         aes(
            x=xmin,
            xend=xend,
            y=y,
            yend=y),
         colour="deeppink4",
         data=guidesDF)
      p + theme(legend.position="none")
      
      # label for guides
      guideLabels<-data.frame(
         x=0,
         y=affine(1-guides/vMax),
         label=c(guides-4)
      )
      
      p<-p+geom_text(
         aes(x=x,y=y,label=label),
         data=guideLabels,
         angle=-alphaStart*180/pi,
         hjust=1.4,
         size=10) + theme(legend.position="bottom")
      p
      
      # item labels
      readableAngle<-function(x){
         angle<-x*(-360/totalLength)-alphaStart*90/pi+90                  # angle<-x*(-360/totalLength)-alphaStart*180/pi+90
         angle+ifelse(sign(cos(angle*pi/180))+sign(sin(angle*pi/180))==-2,180,0)
      }
      readableJustification<-function(x){
         angle<-x*(-360/totalLength)-alphaStart*180/pi+90
         ifelse(sign(cos(angle*pi/180))+sign(sin(angle*pi/180))==-2,1,0)
      }
      
      dfItemLabels<-ddply(df,.(item),plyr::summarize,xmin=xmin[1])
      dfItemLabels<-within(dfItemLabels,{
         x<-xmin+binSize/1.2
         angle<-readableAngle(xmin+binSize/1.2)
         hjust<-readableJustification(xmin+binSize/1.5)
      })
      
      # ADDITIONAL CODE VR/TL-----
      dfItemLabels[1,3] <- 1
      dfItemLabels[1,4] <- -16.86
      
      dfItemLabels <- dfItemLabels %>%
         mutate(x=ifelse(hjust==0, x-20, x+15)) %>%
         mutate(angle=ifelse(hjust==0, angle-5, angle-3))
      
      
      p<-p+geom_text(aes(x=x, label=item, angle=angle, hjust=hjust),
                     y=2.1, size=10, vjust=2.1, data=dfItemLabels)
      p
      
      # family labels
      if(familyLabels){
         familyLabelsDF<-aggregate(xmin~family,data=df,FUN=function(s) mean(s+binSize))
         familyLabelsDF<-within(familyLabelsDF,{
            x<-xmin
            angle<-xmin*(-360/totalLength)-alphaStart*180/pi
         })
         
         p<-p+geom_text(
            aes(
               x=x,
               label=family,
               angle=angle),
            data=familyLabelsDF, size=10,
            y=8.4)
      }  
      # empty background and remove guide lines, ticks and labels
      p<-p+theme(
         panel.background=element_blank(),
         axis.title.x=element_blank(),
         axis.title.y=element_blank(),
         panel.grid.major=element_blank(),
         panel.grid.minor=element_blank(),
         axis.text.x=element_blank(),
         axis.text.y=element_blank(),
         axis.ticks=element_blank(),
         panel.border = element_rect(colour = "grey", fill=NA, size=1),
         legend.text = element_text(colour="black", size = 20),
         legend.position="bottom",
         legend.key.size = unit(2, "cm"),
         plot.title = element_text(color="#666666", face="bold", size=25))
      p
      
      # project to polar coordinates
      p<-p+coord_polar(start=alphaStart)
      
      # nice colour scale
      df$score <- as.factor(df$score)
      if(is.null(legLabels)) legLabels <- levels(df$score)
      names(legLabels) <- levels(df$score)
      library(RColorBrewer)
      p<-p+scale_fill_manual(name=df$score, 
                             values = c("No change" =  colNoChange, 
                                        "no change" = colCircle0, 
                                        "Not relevant" =  colNotRelevant,  
                                        "Positive impact" = colPositiveImpact,
                                        "Negative impact" = colNegativeImpact)
                             , lab=legLabels, guide = guide_legend(title = ""))
      p+ ggtitle(title) 
  
  
 }
